# -*- coding: utf-8 -*-
# Copyright 2023 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from google.cloud.servicecontrol import gapic_version as package_version

__version__ = package_version.__version__


from google.cloud.servicecontrol_v1.services.quota_controller.async_client import (
    QuotaControllerAsyncClient,
)
from google.cloud.servicecontrol_v1.services.quota_controller.client import (
    QuotaControllerClient,
)
from google.cloud.servicecontrol_v1.services.service_controller.async_client import (
    ServiceControllerAsyncClient,
)
from google.cloud.servicecontrol_v1.services.service_controller.client import (
    ServiceControllerClient,
)
from google.cloud.servicecontrol_v1.types.check_error import CheckError
from google.cloud.servicecontrol_v1.types.distribution import Distribution
from google.cloud.servicecontrol_v1.types.http_request import HttpRequest
from google.cloud.servicecontrol_v1.types.log_entry import (
    LogEntry,
    LogEntryOperation,
    LogEntrySourceLocation,
)
from google.cloud.servicecontrol_v1.types.metric_value import (
    MetricValue,
    MetricValueSet,
)
from google.cloud.servicecontrol_v1.types.operation import Operation
from google.cloud.servicecontrol_v1.types.quota_controller import (
    AllocateQuotaRequest,
    AllocateQuotaResponse,
    QuotaError,
    QuotaOperation,
)
from google.cloud.servicecontrol_v1.types.service_controller import (
    CheckRequest,
    CheckResponse,
    ReportRequest,
    ReportResponse,
)

__all__ = (
    "QuotaControllerClient",
    "QuotaControllerAsyncClient",
    "ServiceControllerClient",
    "ServiceControllerAsyncClient",
    "CheckError",
    "Distribution",
    "HttpRequest",
    "LogEntry",
    "LogEntryOperation",
    "LogEntrySourceLocation",
    "MetricValue",
    "MetricValueSet",
    "Operation",
    "AllocateQuotaRequest",
    "AllocateQuotaResponse",
    "QuotaError",
    "QuotaOperation",
    "CheckRequest",
    "CheckResponse",
    "ReportRequest",
    "ReportResponse",
)



# -*- coding: utf-8 -*-
# Copyright 2023 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableMapping, MutableSequence

from google.protobuf import any_pb2  # type: ignore
from google.protobuf import timestamp_pb2  # type: ignore
import proto  # type: ignore

from google.cloud.servicecontrol_v1.types import log_entry, metric_value

__protobuf__ = proto.module(
    package="google.api.servicecontrol.v1",
    manifest={
        "Operation",
    },
)


class Operation(proto.Message):
    r"""Represents information regarding an operation.

    Attributes:
        operation_id (str):
            Identity of the operation. This must be
            unique within the scope of the service that
            generated the operation. If the service calls
            Check() and Report() on the same operation, the
            two calls should carry the same id.

            UUID version 4 is recommended, though not
            required. In scenarios where an operation is
            computed from existing information and an
            idempotent id is desirable for deduplication
            purpose, UUID version 5 is recommended. See RFC
            4122 for details.
        operation_name (str):
            Fully qualified name of the operation.
            Reserved for future use.
        consumer_id (str):
            Identity of the consumer who is using the service. This
            field should be filled in for the operations initiated by a
            consumer, but not for service-initiated operations that are
            not related to a specific consumer.

            -  This can be in one of the following formats:

               -  project:PROJECT_ID,
               -  project\ ``_``\ number:PROJECT_NUMBER,
               -  projects/PROJECT_ID or PROJECT_NUMBER,
               -  folders/FOLDER_NUMBER,
               -  organizations/ORGANIZATION_NUMBER,
               -  api\ ``_``\ key:API_KEY.
        start_time (google.protobuf.timestamp_pb2.Timestamp):
            Required. Start time of the operation.
        end_time (google.protobuf.timestamp_pb2.Timestamp):
            End time of the operation. Required when the operation is
            used in
            [ServiceController.Report][google.api.servicecontrol.v1.ServiceController.Report],
            but optional when the operation is used in
            [ServiceController.Check][google.api.servicecontrol.v1.ServiceController.Check].
        labels (MutableMapping[str, str]):
            Labels describing the operation. Only the following labels
            are allowed:

            -  Labels describing monitored resources as defined in the
               service configuration.
            -  Default labels of metric values. When specified, labels
               defined in the metric value override these default.
            -  The following labels defined by Google Cloud Platform:

               -  ``cloud.googleapis.com/location`` describing the
                  location where the operation happened,
               -  ``servicecontrol.googleapis.com/user_agent``
                  describing the user agent of the API request,
               -  ``servicecontrol.googleapis.com/service_agent``
                  describing the service used to handle the API request
                  (e.g. ESP),
               -  ``servicecontrol.googleapis.com/platform`` describing
                  the platform where the API is served, such as App
                  Engine, Compute Engine, or Kubernetes Engine.
        metric_value_sets (MutableSequence[google.cloud.servicecontrol_v1.types.MetricValueSet]):
            Represents information about this operation.
            Each MetricValueSet corresponds to a metric
            defined in the service configuration. The data
            type used in the MetricValueSet must agree with
            the data type specified in the metric
            definition.

            Within a single operation, it is not allowed to
            have more than one MetricValue instances that
            have the same metric names and identical label
            value combinations. If a request has such
            duplicated MetricValue instances, the entire
            request is rejected with an invalid argument
            error.
        log_entries (MutableSequence[google.cloud.servicecontrol_v1.types.LogEntry]):
            Represents information to be logged.
        importance (google.cloud.servicecontrol_v1.types.Operation.Importance):
            DO NOT USE. This is an experimental field.
        extensions (MutableSequence[google.protobuf.any_pb2.Any]):
            Unimplemented.
    """

    class Importance(proto.Enum):
        r"""Defines the importance of the data contained in the
        operation.

        Values:
            LOW (0):
                Allows data caching, batching, and
                aggregation. It provides higher performance with
                higher data loss risk.
            HIGH (1):
                Disables data aggregation to minimize data
                loss. It is for operations that contains
                significant monetary value or audit trail. This
                feature only applies to the client libraries.
        """
        LOW = 0
        HIGH = 1

    operation_id: str = proto.Field(
        proto.STRING,
        number=1,
    )
    operation_name: str = proto.Field(
        proto.STRING,
        number=2,
    )
    consumer_id: str = proto.Field(
        proto.STRING,
        number=3,
    )
    start_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=4,
        message=timestamp_pb2.Timestamp,
    )
    end_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=5,
        message=timestamp_pb2.Timestamp,
    )
    labels: MutableMapping[str, str] = proto.MapField(
        proto.STRING,
        proto.STRING,
        number=6,
    )
    metric_value_sets: MutableSequence[
        metric_value.MetricValueSet
    ] = proto.RepeatedField(
        proto.MESSAGE,
        number=7,
        message=metric_value.MetricValueSet,
    )
    log_entries: MutableSequence[log_entry.LogEntry] = proto.RepeatedField(
        proto.MESSAGE,
        number=8,
        message=log_entry.LogEntry,
    )
    importance: Importance = proto.Field(
        proto.ENUM,
        number=11,
        enum=Importance,
    )
    extensions: MutableSequence[any_pb2.Any] = proto.RepeatedField(
        proto.MESSAGE,
        number=16,
        message=any_pb2.Any,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2023 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

import dataclasses
import json  # type: ignore
import re
from typing import Any, Callable, Dict, List, Optional, Sequence, Tuple, Union
import warnings

from google.api_core import gapic_v1, path_template, rest_helpers, rest_streaming
from google.api_core import exceptions as core_exceptions
from google.api_core import retry as retries
from google.auth import credentials as ga_credentials  # type: ignore
from google.auth.transport.grpc import SslCredentials  # type: ignore
from google.auth.transport.requests import AuthorizedSession  # type: ignore
from google.protobuf import json_format
import grpc  # type: ignore
from requests import __version__ as requests_version

try:
    OptionalRetry = Union[retries.Retry, gapic_v1.method._MethodDefault]
except AttributeError:  # pragma: NO COVER
    OptionalRetry = Union[retries.Retry, object]  # type: ignore


from google.cloud.servicecontrol_v2.types import service_controller

from .base import DEFAULT_CLIENT_INFO as BASE_DEFAULT_CLIENT_INFO
from .base import ServiceControllerTransport

DEFAULT_CLIENT_INFO = gapic_v1.client_info.ClientInfo(
    gapic_version=BASE_DEFAULT_CLIENT_INFO.gapic_version,
    grpc_version=None,
    rest_version=requests_version,
)


class ServiceControllerRestInterceptor:
    """Interceptor for ServiceController.

    Interceptors are used to manipulate requests, request metadata, and responses
    in arbitrary ways.
    Example use cases include:
    * Logging
    * Verifying requests according to service or custom semantics
    * Stripping extraneous information from responses

    These use cases and more can be enabled by injecting an
    instance of a custom subclass when constructing the ServiceControllerRestTransport.

    .. code-block:: python
        class MyCustomServiceControllerInterceptor(ServiceControllerRestInterceptor):
            def pre_check(self, request, metadata):
                logging.log(f"Received request: {request}")
                return request, metadata

            def post_check(self, response):
                logging.log(f"Received response: {response}")
                return response

            def pre_report(self, request, metadata):
                logging.log(f"Received request: {request}")
                return request, metadata

            def post_report(self, response):
                logging.log(f"Received response: {response}")
                return response

        transport = ServiceControllerRestTransport(interceptor=MyCustomServiceControllerInterceptor())
        client = ServiceControllerClient(transport=transport)


    """

    def pre_check(
        self,
        request: service_controller.CheckRequest,
        metadata: Sequence[Tuple[str, str]],
    ) -> Tuple[service_controller.CheckRequest, Sequence[Tuple[str, str]]]:
        """Pre-rpc interceptor for check

        Override in a subclass to manipulate the request or metadata
        before they are sent to the ServiceController server.
        """
        return request, metadata

    def post_check(
        self, response: service_controller.CheckResponse
    ) -> service_controller.CheckResponse:
        """Post-rpc interceptor for check

        Override in a subclass to manipulate the response
        after it is returned by the ServiceController server but before
        it is returned to user code.
        """
        return response

    def pre_report(
        self,
        request: service_controller.ReportRequest,
        metadata: Sequence[Tuple[str, str]],
    ) -> Tuple[service_controller.ReportRequest, Sequence[Tuple[str, str]]]:
        """Pre-rpc interceptor for report

        Override in a subclass to manipulate the request or metadata
        before they are sent to the ServiceController server.
        """
        return request, metadata

    def post_report(
        self, response: service_controller.ReportResponse
    ) -> service_controller.ReportResponse:
        """Post-rpc interceptor for report

        Override in a subclass to manipulate the response
        after it is returned by the ServiceController server but before
        it is returned to user code.
        """
        return response


@dataclasses.dataclass
class ServiceControllerRestStub:
    _session: AuthorizedSession
    _host: str
    _interceptor: ServiceControllerRestInterceptor


class ServiceControllerRestTransport(ServiceControllerTransport):
    """REST backend transport for ServiceController.

    `Service Control API
    v2 <https://cloud.google.com/service-infrastructure/docs/service-control/access-control>`__

    Private Preview. This feature is only available for approved
    services.

    This API provides admission control and telemetry reporting for
    services that are integrated with `Service
    Infrastructure <https://cloud.google.com/service-infrastructure>`__.

    This class defines the same methods as the primary client, so the
    primary client can load the underlying transport implementation
    and call it.

    It sends JSON representations of protocol buffers over HTTP/1.1

    """

    def __init__(
        self,
        *,
        host: str = "servicecontrol.googleapis.com",
        credentials: Optional[ga_credentials.Credentials] = None,
        credentials_file: Optional[str] = None,
        scopes: Optional[Sequence[str]] = None,
        client_cert_source_for_mtls: Optional[Callable[[], Tuple[bytes, bytes]]] = None,
        quota_project_id: Optional[str] = None,
        client_info: gapic_v1.client_info.ClientInfo = DEFAULT_CLIENT_INFO,
        always_use_jwt_access: Optional[bool] = False,
        url_scheme: str = "https",
        interceptor: Optional[ServiceControllerRestInterceptor] = None,
        api_audience: Optional[str] = None,
    ) -> None:
        """Instantiate the transport.

        Args:
            host (Optional[str]):
                 The hostname to connect to.
            credentials (Optional[google.auth.credentials.Credentials]): The
                authorization credentials to attach to requests. These
                credentials identify the application to the service; if none
                are specified, the client will attempt to ascertain the
                credentials from the environment.

            credentials_file (Optional[str]): A file with credentials that can
                be loaded with :func:`google.auth.load_credentials_from_file`.
                This argument is ignored if ``channel`` is provided.
            scopes (Optional(Sequence[str])): A list of scopes. This argument is
                ignored if ``channel`` is provided.
            client_cert_source_for_mtls (Callable[[], Tuple[bytes, bytes]]): Client
                certificate to configure mutual TLS HTTP channel. It is ignored
                if ``channel`` is provided.
            quota_project_id (Optional[str]): An optional project to use for billing
                and quota.
            client_info (google.api_core.gapic_v1.client_info.ClientInfo):
                The client info used to send a user-agent string along with
                API requests. If ``None``, then default info will be used.
                Generally, you only need to set this if you are developing
                your own client library.
            always_use_jwt_access (Optional[bool]): Whether self signed JWT should
                be used for service account credentials.
            url_scheme: the protocol scheme for the API endpoint.  Normally
                "https", but for testing or local servers,
                "http" can be specified.
        """
        # Run the base constructor
        # TODO(yon-mg): resolve other ctor params i.e. scopes, quota, etc.
        # TODO: When custom host (api_endpoint) is set, `scopes` must *also* be set on the
        # credentials object
        maybe_url_match = re.match("^(?P<scheme>http(?:s)?://)?(?P<host>.*)$", host)
        if maybe_url_match is None:
            raise ValueError(
                f"Unexpected hostname structure: {host}"
            )  # pragma: NO COVER

        url_match_items = maybe_url_match.groupdict()

        host = f"{url_scheme}://{host}" if not url_match_items["scheme"] else host

        super().__init__(
            host=host,
            credentials=credentials,
            client_info=client_info,
            always_use_jwt_access=always_use_jwt_access,
            api_audience=api_audience,
        )
        self._session = AuthorizedSession(
            self._credentials, default_host=self.DEFAULT_HOST
        )
        if client_cert_source_for_mtls:
            self._session.configure_mtls_channel(client_cert_source_for_mtls)
        self._interceptor = interceptor or ServiceControllerRestInterceptor()
        self._prep_wrapped_messages(client_info)

    class _Check(ServiceControllerRestStub):
        def __hash__(self):
            return hash("Check")

        def __call__(
            self,
            request: service_controller.CheckRequest,
            *,
            retry: OptionalRetry = gapic_v1.method.DEFAULT,
            timeout: Optional[float] = None,
            metadata: Sequence[Tuple[str, str]] = (),
        ) -> service_controller.CheckResponse:
            r"""Call the check method over HTTP.

            Args:
                request (~.service_controller.CheckRequest):
                    The request object. Request message for the Check method.
                retry (google.api_core.retry.Retry): Designation of what errors, if any,
                    should be retried.
                timeout (float): The timeout for this request.
                metadata (Sequence[Tuple[str, str]]): Strings which should be
                    sent along with the request as metadata.

            Returns:
                ~.service_controller.CheckResponse:
                    Response message for the Check
                method.

            """

            http_options: List[Dict[str, str]] = [
                {
                    "method": "post",
                    "uri": "/v2/services/{service_name}:check",
                    "body": "*",
                },
            ]
            request, metadata = self._interceptor.pre_check(request, metadata)
            pb_request = service_controller.CheckRequest.pb(request)
            transcoded_request = path_template.transcode(http_options, pb_request)

            # Jsonify the request body

            body = json_format.MessageToJson(
                transcoded_request["body"],
                including_default_value_fields=False,
                use_integers_for_enums=True,
            )
            uri = transcoded_request["uri"]
            method = transcoded_request["method"]

            # Jsonify the query params
            query_params = json.loads(
                json_format.MessageToJson(
                    transcoded_request["query_params"],
                    including_default_value_fields=False,
                    use_integers_for_enums=True,
                )
            )

            query_params["$alt"] = "json;enum-encoding=int"

            # Send the request
            headers = dict(metadata)
            headers["Content-Type"] = "application/json"
            response = getattr(self._session, method)(
                "{host}{uri}".format(host=self._host, uri=uri),
                timeout=timeout,
                headers=headers,
                params=rest_helpers.flatten_query_params(query_params, strict=True),
                data=body,
            )

            # In case of error, raise the appropriate core_exceptions.GoogleAPICallError exception
            # subclass.
            if response.status_code >= 400:
                raise core_exceptions.from_http_response(response)

            # Return the response
            resp = service_controller.CheckResponse()
            pb_resp = service_controller.CheckResponse.pb(resp)

            json_format.Parse(response.content, pb_resp, ignore_unknown_fields=True)
            resp = self._interceptor.post_check(resp)
            return resp

    class _Report(ServiceControllerRestStub):
        def __hash__(self):
            return hash("Report")

        def __call__(
            self,
            request: service_controller.ReportRequest,
            *,
            retry: OptionalRetry = gapic_v1.method.DEFAULT,
            timeout: Optional[float] = None,
            metadata: Sequence[Tuple[str, str]] = (),
        ) -> service_controller.ReportResponse:
            r"""Call the report method over HTTP.

            Args:
                request (~.service_controller.ReportRequest):
                    The request object. Request message for the Report
                method.
                retry (google.api_core.retry.Retry): Designation of what errors, if any,
                    should be retried.
                timeout (float): The timeout for this request.
                metadata (Sequence[Tuple[str, str]]): Strings which should be
                    sent along with the request as metadata.

            Returns:
                ~.service_controller.ReportResponse:
                    Response message for the Report
                method. If the request contains any
                invalid data, the server returns an RPC
                error.

            """

            http_options: List[Dict[str, str]] = [
                {
                    "method": "post",
                    "uri": "/v2/services/{service_name}:report",
                    "body": "*",
                },
            ]
            request, metadata = self._interceptor.pre_report(request, metadata)
            pb_request = service_controller.ReportRequest.pb(request)
            transcoded_request = path_template.transcode(http_options, pb_request)

            # Jsonify the request body

            body = json_format.MessageToJson(
                transcoded_request["body"],
                including_default_value_fields=False,
                use_integers_for_enums=True,
            )
            uri = transcoded_request["uri"]
            method = transcoded_request["method"]

            # Jsonify the query params
            query_params = json.loads(
                json_format.MessageToJson(
                    transcoded_request["query_params"],
                    including_default_value_fields=False,
                    use_integers_for_enums=True,
                )
            )

            query_params["$alt"] = "json;enum-encoding=int"

            # Send the request
            headers = dict(metadata)
            headers["Content-Type"] = "application/json"
            response = getattr(self._session, method)(
                "{host}{uri}".format(host=self._host, uri=uri),
                timeout=timeout,
                headers=headers,
                params=rest_helpers.flatten_query_params(query_params, strict=True),
                data=body,
            )

            # In case of error, raise the appropriate core_exceptions.GoogleAPICallError exception
            # subclass.
            if response.status_code >= 400:
                raise core_exceptions.from_http_response(response)

            # Return the response
            resp = service_controller.ReportResponse()
            pb_resp = service_controller.ReportResponse.pb(resp)

            json_format.Parse(response.content, pb_resp, ignore_unknown_fields=True)
            resp = self._interceptor.post_report(resp)
            return resp

    @property
    def check(
        self,
    ) -> Callable[[service_controller.CheckRequest], service_controller.CheckResponse]:
        # The return type is fine, but mypy isn't sophisticated enough to determine what's going on here.
        # In C++ this would require a dynamic_cast
        return self._Check(self._session, self._host, self._interceptor)  # type: ignore

    @property
    def report(
        self,
    ) -> Callable[
        [service_controller.ReportRequest], service_controller.ReportResponse
    ]:
        # The return type is fine, but mypy isn't sophisticated enough to determine what's going on here.
        # In C++ this would require a dynamic_cast
        return self._Report(self._session, self._host, self._interceptor)  # type: ignore

    @property
    def kind(self) -> str:
        return "rest"

    def close(self):
        self._session.close()


__all__ = ("ServiceControllerRestTransport",)



# -*- coding: utf-8 -*-
# Copyright 2023 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableMapping, MutableSequence

from google.api import distribution_pb2  # type: ignore
import proto  # type: ignore

__protobuf__ = proto.module(
    package="google.api.servicecontrol.v1",
    manifest={
        "Distribution",
    },
)


class Distribution(proto.Message):
    r"""Distribution represents a frequency distribution of double-valued
    sample points. It contains the size of the population of sample
    points plus additional optional information:

    -  the arithmetic mean of the samples
    -  the minimum and maximum of the samples
    -  the sum-squared-deviation of the samples, used to compute
       variance
    -  a histogram of the values of the sample points

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        count (int):
            The total number of samples in the
            distribution. Must be >= 0.
        mean (float):
            The arithmetic mean of the samples in the distribution. If
            ``count`` is zero then this field must be zero.
        minimum (float):
            The minimum of the population of values. Ignored if
            ``count`` is zero.
        maximum (float):
            The maximum of the population of values. Ignored if
            ``count`` is zero.
        sum_of_squared_deviation (float):
            The sum of squared deviations from the mean:
            Sum[i=1..count]((x_i - mean)^2) where each x_i is a sample
            values. If ``count`` is zero then this field must be zero,
            otherwise validation of the request fails.
        bucket_counts (MutableSequence[int]):
            The number of samples in each histogram bucket.
            ``bucket_counts`` are optional. If present, they must sum to
            the ``count`` value.

            The buckets are defined below in ``bucket_option``. There
            are N buckets. ``bucket_counts[0]`` is the number of samples
            in the underflow bucket. ``bucket_counts[1]`` to
            ``bucket_counts[N-1]`` are the numbers of samples in each of
            the finite buckets. And
            ``bucket_counts[N] is the number of samples in the overflow bucket. See the comments of``\ bucket_option\`
            below for more details.

            Any suffix of trailing zeros may be omitted.
        linear_buckets (google.cloud.servicecontrol_v1.types.Distribution.LinearBuckets):
            Buckets with constant width.

            This field is a member of `oneof`_ ``bucket_option``.
        exponential_buckets (google.cloud.servicecontrol_v1.types.Distribution.ExponentialBuckets):
            Buckets with exponentially growing width.

            This field is a member of `oneof`_ ``bucket_option``.
        explicit_buckets (google.cloud.servicecontrol_v1.types.Distribution.ExplicitBuckets):
            Buckets with arbitrary user-provided width.

            This field is a member of `oneof`_ ``bucket_option``.
        exemplars (MutableSequence[google.api.distribution_pb2.Exemplar]):
            Example points. Must be in increasing order of ``value``
            field.
    """

    class LinearBuckets(proto.Message):
        r"""Describing buckets with constant width.

        Attributes:
            num_finite_buckets (int):
                The number of finite buckets. With the underflow and
                overflow buckets, the total number of buckets is
                ``num_finite_buckets`` + 2. See comments on
                ``bucket_options`` for details.
            width (float):
                The i'th linear bucket covers the interval [offset + (i-1)
                \* width, offset + i \* width) where i ranges from 1 to
                num_finite_buckets, inclusive. Must be strictly positive.
            offset (float):
                The i'th linear bucket covers the interval [offset + (i-1)
                \* width, offset + i \* width) where i ranges from 1 to
                num_finite_buckets, inclusive.
        """

        num_finite_buckets: int = proto.Field(
            proto.INT32,
            number=1,
        )
        width: float = proto.Field(
            proto.DOUBLE,
            number=2,
        )
        offset: float = proto.Field(
            proto.DOUBLE,
            number=3,
        )

    class ExponentialBuckets(proto.Message):
        r"""Describing buckets with exponentially growing width.

        Attributes:
            num_finite_buckets (int):
                The number of finite buckets. With the underflow and
                overflow buckets, the total number of buckets is
                ``num_finite_buckets`` + 2. See comments on
                ``bucket_options`` for details.
            growth_factor (float):
                The i'th exponential bucket covers the interval [scale \*
                growth_factor^(i-1), scale \* growth_factor^i) where i
                ranges from 1 to num_finite_buckets inclusive. Must be
                larger than 1.0.
            scale (float):
                The i'th exponential bucket covers the interval [scale \*
                growth_factor^(i-1), scale \* growth_factor^i) where i
                ranges from 1 to num_finite_buckets inclusive. Must be > 0.
        """

        num_finite_buckets: int = proto.Field(
            proto.INT32,
            number=1,
        )
        growth_factor: float = proto.Field(
            proto.DOUBLE,
            number=2,
        )
        scale: float = proto.Field(
            proto.DOUBLE,
            number=3,
        )

    class ExplicitBuckets(proto.Message):
        r"""Describing buckets with arbitrary user-provided width.

        Attributes:
            bounds (MutableSequence[float]):
                'bound' is a list of strictly increasing boundaries between
                buckets. Note that a list of length N-1 defines N buckets
                because of fenceposting. See comments on ``bucket_options``
                for details.

                The i'th finite bucket covers the interval [bound[i-1],
                bound[i]) where i ranges from 1 to bound_size() - 1. Note
                that there are no finite buckets at all if 'bound' only
                contains a single element; in that special case the single
                bound defines the boundary between the underflow and
                overflow buckets.

                bucket number lower bound upper bound i == 0 (underflow)
                -inf bound[i] 0 < i < bound_size() bound[i-1] bound[i] i ==
                bound_size() (overflow) bound[i-1] +inf
        """

        bounds: MutableSequence[float] = proto.RepeatedField(
            proto.DOUBLE,
            number=1,
        )

    count: int = proto.Field(
        proto.INT64,
        number=1,
    )
    mean: float = proto.Field(
        proto.DOUBLE,
        number=2,
    )
    minimum: float = proto.Field(
        proto.DOUBLE,
        number=3,
    )
    maximum: float = proto.Field(
        proto.DOUBLE,
        number=4,
    )
    sum_of_squared_deviation: float = proto.Field(
        proto.DOUBLE,
        number=5,
    )
    bucket_counts: MutableSequence[int] = proto.RepeatedField(
        proto.INT64,
        number=6,
    )
    linear_buckets: LinearBuckets = proto.Field(
        proto.MESSAGE,
        number=7,
        oneof="bucket_option",
        message=LinearBuckets,
    )
    exponential_buckets: ExponentialBuckets = proto.Field(
        proto.MESSAGE,
        number=8,
        oneof="bucket_option",
        message=ExponentialBuckets,
    )
    explicit_buckets: ExplicitBuckets = proto.Field(
        proto.MESSAGE,
        number=9,
        oneof="bucket_option",
        message=ExplicitBuckets,
    )
    exemplars: MutableSequence[
        distribution_pb2.Distribution.Exemplar
    ] = proto.RepeatedField(
        proto.MESSAGE,
        number=10,
        message=distribution_pb2.Distribution.Exemplar,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2023 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableMapping, MutableSequence

from google.protobuf import duration_pb2  # type: ignore
import proto  # type: ignore

__protobuf__ = proto.module(
    package="google.api.servicecontrol.v1",
    manifest={
        "HttpRequest",
    },
)


class HttpRequest(proto.Message):
    r"""A common proto for logging HTTP requests. Only contains
    semantics defined by the HTTP specification. Product-specific
    logging information MUST be defined in a separate message.

    Attributes:
        request_method (str):
            The request method. Examples: ``"GET"``, ``"HEAD"``,
            ``"PUT"``, ``"POST"``.
        request_url (str):
            The scheme (http, https), the host name, the path, and the
            query portion of the URL that was requested. Example:
            ``"http://example.com/some/info?color=red"``.
        request_size (int):
            The size of the HTTP request message in
            bytes, including the request headers and the
            request body.
        status (int):
            The response code indicating the status of
            the response. Examples: 200, 404.
        response_size (int):
            The size of the HTTP response message sent
            back to the client, in bytes, including the
            response headers and the response body.
        user_agent (str):
            The user agent sent by the client. Example:
            ``"Mozilla/4.0 (compatible; MSIE 6.0; Windows 98; Q312461; .NET CLR 1.0.3705)"``.
        remote_ip (str):
            The IP address (IPv4 or IPv6) of the client that issued the
            HTTP request. Examples: ``"192.168.1.1"``,
            ``"FE80::0202:B3FF:FE1E:8329"``.
        server_ip (str):
            The IP address (IPv4 or IPv6) of the origin
            server that the request was sent to.
        referer (str):
            The referer URL of the request, as defined in `HTTP/1.1
            Header Field
            Definitions <http://www.w3.org/Protocols/rfc2616/rfc2616-sec14.html>`__.
        latency (google.protobuf.duration_pb2.Duration):
            The request processing latency on the server,
            from the time the request was received until the
            response was sent.
        cache_lookup (bool):
            Whether or not a cache lookup was attempted.
        cache_hit (bool):
            Whether or not an entity was served from
            cache (with or without validation).
        cache_validated_with_origin_server (bool):
            Whether or not the response was validated with the origin
            server before being served from cache. This field is only
            meaningful if ``cache_hit`` is True.
        cache_fill_bytes (int):
            The number of HTTP response bytes inserted
            into cache. Set only when a cache fill was
            attempted.
        protocol (str):
            Protocol used for the request. Examples:
            "HTTP/1.1", "HTTP/2", "websocket".
    """

    request_method: str = proto.Field(
        proto.STRING,
        number=1,
    )
    request_url: str = proto.Field(
        proto.STRING,
        number=2,
    )
    request_size: int = proto.Field(
        proto.INT64,
        number=3,
    )
    status: int = proto.Field(
        proto.INT32,
        number=4,
    )
    response_size: int = proto.Field(
        proto.INT64,
        number=5,
    )
    user_agent: str = proto.Field(
        proto.STRING,
        number=6,
    )
    remote_ip: str = proto.Field(
        proto.STRING,
        number=7,
    )
    server_ip: str = proto.Field(
        proto.STRING,
        number=13,
    )
    referer: str = proto.Field(
        proto.STRING,
        number=8,
    )
    latency: duration_pb2.Duration = proto.Field(
        proto.MESSAGE,
        number=14,
        message=duration_pb2.Duration,
    )
    cache_lookup: bool = proto.Field(
        proto.BOOL,
        number=11,
    )
    cache_hit: bool = proto.Field(
        proto.BOOL,
        number=9,
    )
    cache_validated_with_origin_server: bool = proto.Field(
        proto.BOOL,
        number=10,
    )
    cache_fill_bytes: int = proto.Field(
        proto.INT64,
        number=12,
    )
    protocol: str = proto.Field(
        proto.STRING,
        number=15,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2023 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableMapping, MutableSequence

from google.rpc import status_pb2  # type: ignore
import proto  # type: ignore

from google.cloud.servicecontrol_v1.types import metric_value

__protobuf__ = proto.module(
    package="google.api.servicecontrol.v1",
    manifest={
        "AllocateQuotaRequest",
        "QuotaOperation",
        "AllocateQuotaResponse",
        "QuotaError",
    },
)


class AllocateQuotaRequest(proto.Message):
    r"""Request message for the AllocateQuota method.

    Attributes:
        service_name (str):
            Name of the service as specified in the service
            configuration. For example, ``"pubsub.googleapis.com"``.

            See [google.api.Service][google.api.Service] for the
            definition of a service name.
        allocate_operation (google.cloud.servicecontrol_v1.types.QuotaOperation):
            Operation that describes the quota
            allocation.
        service_config_id (str):
            Specifies which version of service
            configuration should be used to process the
            request. If unspecified or no matching version
            can be found, the latest one will be used.
    """

    service_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    allocate_operation: "QuotaOperation" = proto.Field(
        proto.MESSAGE,
        number=2,
        message="QuotaOperation",
    )
    service_config_id: str = proto.Field(
        proto.STRING,
        number=4,
    )


class QuotaOperation(proto.Message):
    r"""Represents information regarding a quota operation.

    Attributes:
        operation_id (str):
            Identity of the operation. This is expected to be unique
            within the scope of the service that generated the
            operation, and guarantees idempotency in case of retries.

            In order to ensure best performance and latency in the Quota
            backends, operation_ids are optimally associated with time,
            so that related operations can be accessed fast in storage.
            For this reason, the recommended token for services that
            intend to operate at a high QPS is Unix time in nanos + UUID
        method_name (str):
            Fully qualified name of the API method for which this quota
            operation is requested. This name is used for matching quota
            rules or metric rules and billing status rules defined in
            service configuration.

            This field should not be set if any of the following is
            true: (1) the quota operation is performed on non-API
            resources. (2) quota_metrics is set because the caller is
            doing quota override.

            Example of an RPC method name:
            google.example.library.v1.LibraryService.CreateShelf
        consumer_id (str):
            Identity of the consumer for whom this quota operation is
            being performed.

            This can be in one of the following formats:
            project:<project_id>, project_number:<project_number>,
            api_key:<api_key>.
        labels (MutableMapping[str, str]):
            Labels describing the operation.
        quota_metrics (MutableSequence[google.cloud.servicecontrol_v1.types.MetricValueSet]):
            Represents information about this operation. Each
            MetricValueSet corresponds to a metric defined in the
            service configuration. The data type used in the
            MetricValueSet must agree with the data type specified in
            the metric definition.

            Within a single operation, it is not allowed to have more
            than one MetricValue instances that have the same metric
            names and identical label value combinations. If a request
            has such duplicated MetricValue instances, the entire
            request is rejected with an invalid argument error.

            This field is mutually exclusive with method_name.
        quota_mode (google.cloud.servicecontrol_v1.types.QuotaOperation.QuotaMode):
            Quota mode for this operation.
    """

    class QuotaMode(proto.Enum):
        r"""Supported quota modes.

        Values:
            UNSPECIFIED (0):
                Guard against implicit default. Must not be
                used.
            NORMAL (1):
                For AllocateQuota request, allocates quota
                for the amount specified in the service
                configuration or specified using the quota
                metrics. If the amount is higher than the
                available quota, allocation error will be
                returned and no quota will be allocated.
                If multiple quotas are part of the request, and
                one fails, none of the quotas are allocated or
                released.
            BEST_EFFORT (2):
                The operation allocates quota for the amount specified in
                the service configuration or specified using the quota
                metrics. If the amount is higher than the available quota,
                request does not fail but all available quota will be
                allocated. For rate quota, BEST_EFFORT will continue to
                deduct from other groups even if one does not have enough
                quota. For allocation, it will find the minimum available
                amount across all groups and deduct that amount from all the
                affected groups.
            CHECK_ONLY (3):
                For AllocateQuota request, only checks if
                there is enough quota available and does not
                change the available quota. No lock is placed on
                the available quota either.
            QUERY_ONLY (4):
                Unimplemented. When used in
                AllocateQuotaRequest, this returns the effective
                quota limit(s) in the response, and no quota
                check will be performed. Not supported for other
                requests, and even for AllocateQuotaRequest,
                this is currently supported only for allowlisted
                services.
            ADJUST_ONLY (5):
                The operation allocates quota for the amount
                specified in the service configuration or
                specified using the quota metrics. If the
                requested amount is higher than the available
                quota, request does not fail and remaining quota
                would become negative (going over the limit).
                Not supported for Rate Quota.
        """
        UNSPECIFIED = 0
        NORMAL = 1
        BEST_EFFORT = 2
        CHECK_ONLY = 3
        QUERY_ONLY = 4
        ADJUST_ONLY = 5

    operation_id: str = proto.Field(
        proto.STRING,
        number=1,
    )
    method_name: str = proto.Field(
        proto.STRING,
        number=2,
    )
    consumer_id: str = proto.Field(
        proto.STRING,
        number=3,
    )
    labels: MutableMapping[str, str] = proto.MapField(
        proto.STRING,
        proto.STRING,
        number=4,
    )
    quota_metrics: MutableSequence[metric_value.MetricValueSet] = proto.RepeatedField(
        proto.MESSAGE,
        number=5,
        message=metric_value.MetricValueSet,
    )
    quota_mode: QuotaMode = proto.Field(
        proto.ENUM,
        number=6,
        enum=QuotaMode,
    )


class AllocateQuotaResponse(proto.Message):
    r"""Response message for the AllocateQuota method.

    Attributes:
        operation_id (str):
            The same operation_id value used in the
            AllocateQuotaRequest. Used for logging and diagnostics
            purposes.
        allocate_errors (MutableSequence[google.cloud.servicecontrol_v1.types.QuotaError]):
            Indicates the decision of the allocate.
        quota_metrics (MutableSequence[google.cloud.servicecontrol_v1.types.MetricValueSet]):
            Quota metrics to indicate the result of allocation.
            Depending on the request, one or more of the following
            metrics will be included:

            1. Per quota group or per quota metric incremental usage
               will be specified using the following delta metric :
               "serviceruntime.googleapis.com/api/consumer/quota_used_count"

            2. The quota limit reached condition will be specified using
               the following boolean metric :
               "serviceruntime.googleapis.com/quota/exceeded".
        service_config_id (str):
            ID of the actual config used to process the
            request.
    """

    operation_id: str = proto.Field(
        proto.STRING,
        number=1,
    )
    allocate_errors: MutableSequence["QuotaError"] = proto.RepeatedField(
        proto.MESSAGE,
        number=2,
        message="QuotaError",
    )
    quota_metrics: MutableSequence[metric_value.MetricValueSet] = proto.RepeatedField(
        proto.MESSAGE,
        number=3,
        message=metric_value.MetricValueSet,
    )
    service_config_id: str = proto.Field(
        proto.STRING,
        number=4,
    )


class QuotaError(proto.Message):
    r"""Represents error information for
    [QuotaOperation][google.api.servicecontrol.v1.QuotaOperation].

    Attributes:
        code (google.cloud.servicecontrol_v1.types.QuotaError.Code):
            Error code.
        subject (str):
            Subject to whom this error applies. See the
            specific enum for more details on this field.
            For example, "clientip:<ip address of client>"
            or "project:<Google developer project id>".
        description (str):
            Free-form text that provides details on the
            cause of the error.
        status (google.rpc.status_pb2.Status):
            Contains additional information about the quota error. If
            available, ``status.code`` will be non zero.
    """

    class Code(proto.Enum):
        r"""Error codes related to project config validations are deprecated
        since the quota controller methods do not perform these validations.
        Instead services have to call the Check method, without
        quota_properties field, to perform these validations before calling
        the quota controller methods. These methods check only for project
        deletion to be wipe out compliant.

        Values:
            UNSPECIFIED (0):
                This is never used.
            RESOURCE_EXHAUSTED (8):
                Quota allocation failed. Same as
                [google.rpc.Code.RESOURCE_EXHAUSTED][google.rpc.Code.RESOURCE_EXHAUSTED].
            BILLING_NOT_ACTIVE (107):
                Consumer cannot access the service because
                the service requires active billing.
            PROJECT_DELETED (108):
                Consumer's project has been marked as deleted
                (soft deletion).
            API_KEY_INVALID (105):
                Specified API key is invalid.
            API_KEY_EXPIRED (112):
                Specified API Key has expired.
        """
        UNSPECIFIED = 0
        RESOURCE_EXHAUSTED = 8
        BILLING_NOT_ACTIVE = 107
        PROJECT_DELETED = 108
        API_KEY_INVALID = 105
        API_KEY_EXPIRED = 112

    code: Code = proto.Field(
        proto.ENUM,
        number=1,
        enum=Code,
    )
    subject: str = proto.Field(
        proto.STRING,
        number=2,
    )
    description: str = proto.Field(
        proto.STRING,
        number=3,
    )
    status: status_pb2.Status = proto.Field(
        proto.MESSAGE,
        number=4,
        message=status_pb2.Status,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2023 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableMapping, MutableSequence

from google.rpc import status_pb2  # type: ignore
import proto  # type: ignore

__protobuf__ = proto.module(
    package="google.api.servicecontrol.v1",
    manifest={
        "CheckError",
    },
)


class CheckError(proto.Message):
    r"""Defines the errors to be returned in
    [google.api.servicecontrol.v1.CheckResponse.check_errors][google.api.servicecontrol.v1.CheckResponse.check_errors].

    Attributes:
        code (google.cloud.servicecontrol_v1.types.CheckError.Code):
            The error code.
        subject (str):
            Subject to whom this error applies. See the
            specific code enum for more details on this
            field. For example:

            - "project:<project-id or project-number>"
            - "folder:<folder-id>"
            - "organization:<organization-id>".
        detail (str):
            Free-form text providing details on the error
            cause of the error.
        status (google.rpc.status_pb2.Status):
            Contains public information about the check error. If
            available, ``status.code`` will be non zero and client can
            propagate it out as public error.
    """

    class Code(proto.Enum):
        r"""Error codes for Check responses.

        Values:
            ERROR_CODE_UNSPECIFIED (0):
                This is never used in ``CheckResponse``.
            NOT_FOUND (5):
                The consumer's project id, network container, or resource
                container was not found. Same as
                [google.rpc.Code.NOT_FOUND][google.rpc.Code.NOT_FOUND].
            PERMISSION_DENIED (7):
                The consumer doesn't have access to the specified resource.
                Same as
                [google.rpc.Code.PERMISSION_DENIED][google.rpc.Code.PERMISSION_DENIED].
            RESOURCE_EXHAUSTED (8):
                Quota check failed. Same as
                [google.rpc.Code.RESOURCE_EXHAUSTED][google.rpc.Code.RESOURCE_EXHAUSTED].
            SERVICE_NOT_ACTIVATED (104):
                The consumer hasn't activated the service.
            BILLING_DISABLED (107):
                The consumer cannot access the service
                because billing is disabled.
            PROJECT_DELETED (108):
                The consumer's project has been marked as
                deleted (soft deletion).
            PROJECT_INVALID (114):
                The consumer's project number or id does not
                represent a valid project.
            CONSUMER_INVALID (125):
                The input consumer info does not represent a
                valid consumer folder or organization.
            IP_ADDRESS_BLOCKED (109):
                The IP address of the consumer is invalid for
                the specific consumer project.
            REFERER_BLOCKED (110):
                The referer address of the consumer request
                is invalid for the specific consumer project.
            CLIENT_APP_BLOCKED (111):
                The client application of the consumer
                request is invalid for the specific consumer
                project.
            API_TARGET_BLOCKED (122):
                The API targeted by this request is invalid
                for the specified consumer project.
            API_KEY_INVALID (105):
                The consumer's API key is invalid.
            API_KEY_EXPIRED (112):
                The consumer's API Key has expired.
            API_KEY_NOT_FOUND (113):
                The consumer's API Key was not found in
                config record.
            INVALID_CREDENTIAL (123):
                The credential in the request can not be
                verified.
            NAMESPACE_LOOKUP_UNAVAILABLE (300):
                The backend server for looking up project
                id/number is unavailable.
            SERVICE_STATUS_UNAVAILABLE (301):
                The backend server for checking service
                status is unavailable.
            BILLING_STATUS_UNAVAILABLE (302):
                The backend server for checking billing
                status is unavailable.
            CLOUD_RESOURCE_MANAGER_BACKEND_UNAVAILABLE (305):
                Cloud Resource Manager backend server is
                unavailable.
        """
        ERROR_CODE_UNSPECIFIED = 0
        NOT_FOUND = 5
        PERMISSION_DENIED = 7
        RESOURCE_EXHAUSTED = 8
        SERVICE_NOT_ACTIVATED = 104
        BILLING_DISABLED = 107
        PROJECT_DELETED = 108
        PROJECT_INVALID = 114
        CONSUMER_INVALID = 125
        IP_ADDRESS_BLOCKED = 109
        REFERER_BLOCKED = 110
        CLIENT_APP_BLOCKED = 111
        API_TARGET_BLOCKED = 122
        API_KEY_INVALID = 105
        API_KEY_EXPIRED = 112
        API_KEY_NOT_FOUND = 113
        INVALID_CREDENTIAL = 123
        NAMESPACE_LOOKUP_UNAVAILABLE = 300
        SERVICE_STATUS_UNAVAILABLE = 301
        BILLING_STATUS_UNAVAILABLE = 302
        CLOUD_RESOURCE_MANAGER_BACKEND_UNAVAILABLE = 305

    code: Code = proto.Field(
        proto.ENUM,
        number=1,
        enum=Code,
    )
    subject: str = proto.Field(
        proto.STRING,
        number=4,
    )
    detail: str = proto.Field(
        proto.STRING,
        number=2,
    )
    status: status_pb2.Status = proto.Field(
        proto.MESSAGE,
        number=3,
        message=status_pb2.Status,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2023 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableMapping, MutableSequence

from google.rpc import status_pb2  # type: ignore
import proto  # type: ignore

from google.cloud.servicecontrol_v1.types import check_error
from google.cloud.servicecontrol_v1.types import operation as gas_operation

__protobuf__ = proto.module(
    package="google.api.servicecontrol.v1",
    manifest={
        "CheckRequest",
        "CheckResponse",
        "ReportRequest",
        "ReportResponse",
    },
)


class CheckRequest(proto.Message):
    r"""Request message for the Check method.

    Attributes:
        service_name (str):
            The service name as specified in its service configuration.
            For example, ``"pubsub.googleapis.com"``.

            See
            `google.api.Service <https://cloud.google.com/service-management/reference/rpc/google.api#google.api.Service>`__
            for the definition of a service name.
        operation (google.cloud.servicecontrol_v1.types.Operation):
            The operation to be checked.
        service_config_id (str):
            Specifies which version of service
            configuration should be used to process the
            request.

            If unspecified or no matching version can be
            found, the latest one will be used.
    """

    service_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    operation: gas_operation.Operation = proto.Field(
        proto.MESSAGE,
        number=2,
        message=gas_operation.Operation,
    )
    service_config_id: str = proto.Field(
        proto.STRING,
        number=4,
    )


class CheckResponse(proto.Message):
    r"""Response message for the Check method.

    Attributes:
        operation_id (str):
            The same operation_id value used in the
            [CheckRequest][google.api.servicecontrol.v1.CheckRequest].
            Used for logging and diagnostics purposes.
        check_errors (MutableSequence[google.cloud.servicecontrol_v1.types.CheckError]):
            Indicate the decision of the check.

            If no check errors are present, the service
            should process the operation. Otherwise the
            service should use the list of errors to
            determine the appropriate action.
        service_config_id (str):
            The actual config id used to process the
            request.
        service_rollout_id (str):
            The current service rollout id used to
            process the request.
        check_info (google.cloud.servicecontrol_v1.types.CheckResponse.CheckInfo):
            Feedback data returned from the server during
            processing a Check request.
    """

    class CheckInfo(proto.Message):
        r"""Contains additional information about the check operation.

        Attributes:
            unused_arguments (MutableSequence[str]):
                A list of fields and label keys that are
                ignored by the server. The client doesn't need
                to send them for following requests to improve
                performance and allow better aggregation.
            consumer_info (google.cloud.servicecontrol_v1.types.CheckResponse.ConsumerInfo):
                Consumer info of this check.
        """

        unused_arguments: MutableSequence[str] = proto.RepeatedField(
            proto.STRING,
            number=1,
        )
        consumer_info: "CheckResponse.ConsumerInfo" = proto.Field(
            proto.MESSAGE,
            number=2,
            message="CheckResponse.ConsumerInfo",
        )

    class ConsumerInfo(proto.Message):
        r"""``ConsumerInfo`` provides information about the consumer.

        Attributes:
            project_number (int):
                The Google cloud project number, e.g.
                1234567890. A value of 0 indicates no project
                    number is found.

                NOTE: This field is deprecated after we support
                flexible consumer id. New code should not depend
                on this field anymore.
            type_ (google.cloud.servicecontrol_v1.types.CheckResponse.ConsumerInfo.ConsumerType):
                The type of the consumer which should have been defined in
                `Google Resource
                Manager <https://cloud.google.com/resource-manager/>`__.
            consumer_number (int):
                The consumer identity number, can be Google
                cloud project number, folder number or
                organization number e.g. 1234567890. A value of
                0 indicates no consumer number is found.
        """

        class ConsumerType(proto.Enum):
            r"""The type of the consumer as defined in `Google Resource
            Manager <https://cloud.google.com/resource-manager/>`__.

            Values:
                CONSUMER_TYPE_UNSPECIFIED (0):
                    This is never used.
                PROJECT (1):
                    The consumer is a Google Cloud Project.
                FOLDER (2):
                    The consumer is a Google Cloud Folder.
                ORGANIZATION (3):
                    The consumer is a Google Cloud Organization.
                SERVICE_SPECIFIC (4):
                    Service-specific resource container which is
                    defined by the service producer to offer their
                    users the ability to manage service control
                    functionalities at a finer level of granularity
                    than the PROJECT.
            """
            CONSUMER_TYPE_UNSPECIFIED = 0
            PROJECT = 1
            FOLDER = 2
            ORGANIZATION = 3
            SERVICE_SPECIFIC = 4

        project_number: int = proto.Field(
            proto.INT64,
            number=1,
        )
        type_: "CheckResponse.ConsumerInfo.ConsumerType" = proto.Field(
            proto.ENUM,
            number=2,
            enum="CheckResponse.ConsumerInfo.ConsumerType",
        )
        consumer_number: int = proto.Field(
            proto.INT64,
            number=3,
        )

    operation_id: str = proto.Field(
        proto.STRING,
        number=1,
    )
    check_errors: MutableSequence[check_error.CheckError] = proto.RepeatedField(
        proto.MESSAGE,
        number=2,
        message=check_error.CheckError,
    )
    service_config_id: str = proto.Field(
        proto.STRING,
        number=5,
    )
    service_rollout_id: str = proto.Field(
        proto.STRING,
        number=11,
    )
    check_info: CheckInfo = proto.Field(
        proto.MESSAGE,
        number=6,
        message=CheckInfo,
    )


class ReportRequest(proto.Message):
    r"""Request message for the Report method.

    Attributes:
        service_name (str):
            The service name as specified in its service configuration.
            For example, ``"pubsub.googleapis.com"``.

            See
            `google.api.Service <https://cloud.google.com/service-management/reference/rpc/google.api#google.api.Service>`__
            for the definition of a service name.
        operations (MutableSequence[google.cloud.servicecontrol_v1.types.Operation]):
            Operations to be reported.

            Typically the service should report one operation per
            request. Putting multiple operations into a single request
            is allowed, but should be used only when multiple operations
            are natually available at the time of the report.

            There is no limit on the number of operations in the same
            ReportRequest, however the ReportRequest size should be no
            larger than 1MB. See
            [ReportResponse.report_errors][google.api.servicecontrol.v1.ReportResponse.report_errors]
            for partial failure behavior.
        service_config_id (str):
            Specifies which version of service config
            should be used to process the request.

            If unspecified or no matching version can be
            found, the latest one will be used.
    """

    service_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    operations: MutableSequence[gas_operation.Operation] = proto.RepeatedField(
        proto.MESSAGE,
        number=2,
        message=gas_operation.Operation,
    )
    service_config_id: str = proto.Field(
        proto.STRING,
        number=3,
    )


class ReportResponse(proto.Message):
    r"""Response message for the Report method.

    Attributes:
        report_errors (MutableSequence[google.cloud.servicecontrol_v1.types.ReportResponse.ReportError]):
            Partial failures, one for each ``Operation`` in the request
            that failed processing. There are three possible
            combinations of the RPC status:

            1. The combination of a successful RPC status and an empty
               ``report_errors`` list indicates a complete success where
               all ``Operations`` in the request are processed
               successfully.
            2. The combination of a successful RPC status and a
               non-empty ``report_errors`` list indicates a partial
               success where some ``Operations`` in the request
               succeeded. Each ``Operation`` that failed processing has
               a corresponding item in this list.
            3. A failed RPC status indicates a general non-deterministic
               failure. When this happens, it's impossible to know which
               of the 'Operations' in the request succeeded or failed.
        service_config_id (str):
            The actual config id used to process the
            request.
        service_rollout_id (str):
            The current service rollout id used to
            process the request.
    """

    class ReportError(proto.Message):
        r"""Represents the processing error of one
        [Operation][google.api.servicecontrol.v1.Operation] in the request.

        Attributes:
            operation_id (str):
                The
                [Operation.operation_id][google.api.servicecontrol.v1.Operation.operation_id]
                value from the request.
            status (google.rpc.status_pb2.Status):
                Details of the error when processing the
                [Operation][google.api.servicecontrol.v1.Operation].
        """

        operation_id: str = proto.Field(
            proto.STRING,
            number=1,
        )
        status: status_pb2.Status = proto.Field(
            proto.MESSAGE,
            number=2,
            message=status_pb2.Status,
        )

    report_errors: MutableSequence[ReportError] = proto.RepeatedField(
        proto.MESSAGE,
        number=1,
        message=ReportError,
    )
    service_config_id: str = proto.Field(
        proto.STRING,
        number=2,
    )
    service_rollout_id: str = proto.Field(
        proto.STRING,
        number=4,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2023 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableMapping, MutableSequence

from google.rpc import status_pb2  # type: ignore
from google.rpc.context import attribute_context_pb2  # type: ignore
import proto  # type: ignore

__protobuf__ = proto.module(
    package="google.api.servicecontrol.v2",
    manifest={
        "CheckRequest",
        "ResourceInfo",
        "CheckResponse",
        "ReportRequest",
        "ReportResponse",
        "ResourceInfoList",
    },
)


class CheckRequest(proto.Message):
    r"""Request message for the Check method.

    Attributes:
        service_name (str):
            The service name as specified in its service configuration.
            For example, ``"pubsub.googleapis.com"``.

            See
            `google.api.Service <https://cloud.google.com/service-management/reference/rpc/google.api#google.api.Service>`__
            for the definition of a service name.
        service_config_id (str):
            Specifies the version of the service
            configuration that should be used to process the
            request. Must not be empty. Set this field to
            'latest' to specify using the latest
            configuration.
        attributes (google.rpc.context.attribute_context_pb2.AttributeContext):
            Describes attributes about the operation
            being executed by the service.
        resources (MutableSequence[google.cloud.servicecontrol_v2.types.ResourceInfo]):
            Describes the resources and the policies
            applied to each resource.
        flags (str):
            Optional. Contains a comma-separated list of
            flags.
    """

    service_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    service_config_id: str = proto.Field(
        proto.STRING,
        number=2,
    )
    attributes: attribute_context_pb2.AttributeContext = proto.Field(
        proto.MESSAGE,
        number=3,
        message=attribute_context_pb2.AttributeContext,
    )
    resources: MutableSequence["ResourceInfo"] = proto.RepeatedField(
        proto.MESSAGE,
        number=4,
        message="ResourceInfo",
    )
    flags: str = proto.Field(
        proto.STRING,
        number=5,
    )


class ResourceInfo(proto.Message):
    r"""Describes a resource referenced in the request.

    Attributes:
        name (str):
            The name of the resource referenced in the
            request.
        type_ (str):
            The resource type in the format of
            "{service}/{kind}".
        permission (str):
            The resource permission needed for this
            request. The format must be
            "{service}/{plural}.{verb}".
        container (str):
            Optional. The identifier of the container of this resource.
            For Google Cloud APIs, the resource container must be one of
            the following formats: -
            ``projects/<project-id or project-number>`` -
            ``folders/<folder-id>`` -
            ``organizations/<organization-id>`` For the policy
            enforcement on the container level (VPCSC and Location
            Policy check), this field takes precedence on the container
            extracted from name when presents.
        location (str):
            Optional. The location of the resource. The
            value must be a valid zone, region or
            multiregion. For example: "europe-west4" or
            "northamerica-northeast1-a".
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    type_: str = proto.Field(
        proto.STRING,
        number=2,
    )
    permission: str = proto.Field(
        proto.STRING,
        number=3,
    )
    container: str = proto.Field(
        proto.STRING,
        number=4,
    )
    location: str = proto.Field(
        proto.STRING,
        number=5,
    )


class CheckResponse(proto.Message):
    r"""Response message for the Check method.

    Attributes:
        status (google.rpc.status_pb2.Status):
            Operation is allowed when this field is not set. Any
            non-'OK' status indicates a denial;
            [google.rpc.Status.details][google.rpc.Status.details] would
            contain additional details about the denial.
        headers (MutableMapping[str, str]):
            Returns a set of request contexts generated from the
            ``CheckRequest``.
    """

    status: status_pb2.Status = proto.Field(
        proto.MESSAGE,
        number=1,
        message=status_pb2.Status,
    )
    headers: MutableMapping[str, str] = proto.MapField(
        proto.STRING,
        proto.STRING,
        number=2,
    )


class ReportRequest(proto.Message):
    r"""Request message for the Report method.

    Attributes:
        service_name (str):
            The service name as specified in its service configuration.
            For example, ``"pubsub.googleapis.com"``.

            See
            `google.api.Service <https://cloud.google.com/service-management/reference/rpc/google.api#google.api.Service>`__
            for the definition of a service name.
        service_config_id (str):
            Specifies the version of the service
            configuration that should be used to process the
            request. Must not be empty. Set this field to
            'latest' to specify using the latest
            configuration.
        operations (MutableSequence[google.rpc.context.attribute_context_pb2.AttributeContext]):
            Describes the list of operations to be
            reported. Each operation is represented as an
            AttributeContext, and contains all attributes
            around an API access.
    """

    service_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    service_config_id: str = proto.Field(
        proto.STRING,
        number=2,
    )
    operations: MutableSequence[
        attribute_context_pb2.AttributeContext
    ] = proto.RepeatedField(
        proto.MESSAGE,
        number=3,
        message=attribute_context_pb2.AttributeContext,
    )


class ReportResponse(proto.Message):
    r"""Response message for the Report method.
    If the request contains any invalid data, the server returns an
    RPC error.

    """


class ResourceInfoList(proto.Message):
    r"""Message containing resource details in a batch mode.

    Attributes:
        resources (MutableSequence[google.cloud.servicecontrol_v2.types.ResourceInfo]):
            The resource details.
    """

    resources: MutableSequence["ResourceInfo"] = proto.RepeatedField(
        proto.MESSAGE,
        number=1,
        message="ResourceInfo",
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2023 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableMapping, MutableSequence

from google.protobuf import timestamp_pb2  # type: ignore
import proto  # type: ignore

from google.cloud.servicecontrol_v1.types import distribution as gas_distribution

__protobuf__ = proto.module(
    package="google.api.servicecontrol.v1",
    manifest={
        "MetricValue",
        "MetricValueSet",
    },
)


class MetricValue(proto.Message):
    r"""Represents a single metric value.

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        labels (MutableMapping[str, str]):
            The labels describing the metric value. See comments on
            [google.api.servicecontrol.v1.Operation.labels][google.api.servicecontrol.v1.Operation.labels]
            for the overriding relationship. Note that this map must not
            contain monitored resource labels.
        start_time (google.protobuf.timestamp_pb2.Timestamp):
            The start of the time period over which this metric value's
            measurement applies. The time period has different semantics
            for different metric types (cumulative, delta, and gauge).
            See the metric definition documentation in the service
            configuration for details. If not specified,
            [google.api.servicecontrol.v1.Operation.start_time][google.api.servicecontrol.v1.Operation.start_time]
            will be used.
        end_time (google.protobuf.timestamp_pb2.Timestamp):
            The end of the time period over which this metric value's
            measurement applies. If not specified,
            [google.api.servicecontrol.v1.Operation.end_time][google.api.servicecontrol.v1.Operation.end_time]
            will be used.
        bool_value (bool):
            A boolean value.

            This field is a member of `oneof`_ ``value``.
        int64_value (int):
            A signed 64-bit integer value.

            This field is a member of `oneof`_ ``value``.
        double_value (float):
            A double precision floating point value.

            This field is a member of `oneof`_ ``value``.
        string_value (str):
            A text string value.

            This field is a member of `oneof`_ ``value``.
        distribution_value (google.cloud.servicecontrol_v1.types.Distribution):
            A distribution value.

            This field is a member of `oneof`_ ``value``.
    """

    labels: MutableMapping[str, str] = proto.MapField(
        proto.STRING,
        proto.STRING,
        number=1,
    )
    start_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=2,
        message=timestamp_pb2.Timestamp,
    )
    end_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=3,
        message=timestamp_pb2.Timestamp,
    )
    bool_value: bool = proto.Field(
        proto.BOOL,
        number=4,
        oneof="value",
    )
    int64_value: int = proto.Field(
        proto.INT64,
        number=5,
        oneof="value",
    )
    double_value: float = proto.Field(
        proto.DOUBLE,
        number=6,
        oneof="value",
    )
    string_value: str = proto.Field(
        proto.STRING,
        number=7,
        oneof="value",
    )
    distribution_value: gas_distribution.Distribution = proto.Field(
        proto.MESSAGE,
        number=8,
        oneof="value",
        message=gas_distribution.Distribution,
    )


class MetricValueSet(proto.Message):
    r"""Represents a set of metric values in the same metric.
    Each metric value in the set should have a unique combination of
    start time, end time, and label values.

    Attributes:
        metric_name (str):
            The metric name defined in the service
            configuration.
        metric_values (MutableSequence[google.cloud.servicecontrol_v1.types.MetricValue]):
            The values in this metric.
    """

    metric_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    metric_values: MutableSequence["MetricValue"] = proto.RepeatedField(
        proto.MESSAGE,
        number=2,
        message="MetricValue",
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2023 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableMapping, MutableSequence

from google.logging.type import log_severity_pb2  # type: ignore
from google.protobuf import any_pb2  # type: ignore
from google.protobuf import struct_pb2  # type: ignore
from google.protobuf import timestamp_pb2  # type: ignore
import proto  # type: ignore

from google.cloud.servicecontrol_v1.types import http_request as gas_http_request

__protobuf__ = proto.module(
    package="google.api.servicecontrol.v1",
    manifest={
        "LogEntry",
        "LogEntryOperation",
        "LogEntrySourceLocation",
    },
)


class LogEntry(proto.Message):
    r"""An individual log entry.

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        name (str):
            Required. The log to which this log entry belongs. Examples:
            ``"syslog"``, ``"book_log"``.
        timestamp (google.protobuf.timestamp_pb2.Timestamp):
            The time the event described by the log entry
            occurred. If omitted, defaults to operation
            start time.
        severity (google.logging.type.log_severity_pb2.LogSeverity):
            The severity of the log entry. The default value is
            ``LogSeverity.DEFAULT``.
        http_request (google.cloud.servicecontrol_v1.types.HttpRequest):
            Optional. Information about the HTTP request
            associated with this log entry, if applicable.
        trace (str):
            Optional. Resource name of the trace associated with the log
            entry, if any. If this field contains a relative resource
            name, you can assume the name is relative to
            ``//tracing.googleapis.com``. Example:
            ``projects/my-projectid/traces/06796866738c859f2f19b7cfb3214824``
        insert_id (str):
            A unique ID for the log entry used for deduplication. If
            omitted, the implementation will generate one based on
            operation_id.
        labels (MutableMapping[str, str]):
            A set of user-defined (key, value) data that
            provides additional information about the log
            entry.
        proto_payload (google.protobuf.any_pb2.Any):
            The log entry payload, represented as a protocol buffer that
            is expressed as a JSON object. The only accepted type
            currently is [AuditLog][google.cloud.audit.AuditLog].

            This field is a member of `oneof`_ ``payload``.
        text_payload (str):
            The log entry payload, represented as a
            Unicode string (UTF-8).

            This field is a member of `oneof`_ ``payload``.
        struct_payload (google.protobuf.struct_pb2.Struct):
            The log entry payload, represented as a
            structure that is expressed as a JSON object.

            This field is a member of `oneof`_ ``payload``.
        operation (google.cloud.servicecontrol_v1.types.LogEntryOperation):
            Optional. Information about an operation
            associated with the log entry, if applicable.
        source_location (google.cloud.servicecontrol_v1.types.LogEntrySourceLocation):
            Optional. Source code location information
            associated with the log entry, if any.
    """

    name: str = proto.Field(
        proto.STRING,
        number=10,
    )
    timestamp: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=11,
        message=timestamp_pb2.Timestamp,
    )
    severity: log_severity_pb2.LogSeverity = proto.Field(
        proto.ENUM,
        number=12,
        enum=log_severity_pb2.LogSeverity,
    )
    http_request: gas_http_request.HttpRequest = proto.Field(
        proto.MESSAGE,
        number=14,
        message=gas_http_request.HttpRequest,
    )
    trace: str = proto.Field(
        proto.STRING,
        number=15,
    )
    insert_id: str = proto.Field(
        proto.STRING,
        number=4,
    )
    labels: MutableMapping[str, str] = proto.MapField(
        proto.STRING,
        proto.STRING,
        number=13,
    )
    proto_payload: any_pb2.Any = proto.Field(
        proto.MESSAGE,
        number=2,
        oneof="payload",
        message=any_pb2.Any,
    )
    text_payload: str = proto.Field(
        proto.STRING,
        number=3,
        oneof="payload",
    )
    struct_payload: struct_pb2.Struct = proto.Field(
        proto.MESSAGE,
        number=6,
        oneof="payload",
        message=struct_pb2.Struct,
    )
    operation: "LogEntryOperation" = proto.Field(
        proto.MESSAGE,
        number=16,
        message="LogEntryOperation",
    )
    source_location: "LogEntrySourceLocation" = proto.Field(
        proto.MESSAGE,
        number=17,
        message="LogEntrySourceLocation",
    )


class LogEntryOperation(proto.Message):
    r"""Additional information about a potentially long-running
    operation with which a log entry is associated.

    Attributes:
        id (str):
            Optional. An arbitrary operation identifier.
            Log entries with the same identifier are assumed
            to be part of the same operation.
        producer (str):
            Optional. An arbitrary producer identifier. The combination
            of ``id`` and ``producer`` must be globally unique. Examples
            for ``producer``: ``"MyDivision.MyBigCompany.com"``,
            ``"github.com/MyProject/MyApplication"``.
        first (bool):
            Optional. Set this to True if this is the
            first log entry in the operation.
        last (bool):
            Optional. Set this to True if this is the
            last log entry in the operation.
    """

    id: str = proto.Field(
        proto.STRING,
        number=1,
    )
    producer: str = proto.Field(
        proto.STRING,
        number=2,
    )
    first: bool = proto.Field(
        proto.BOOL,
        number=3,
    )
    last: bool = proto.Field(
        proto.BOOL,
        number=4,
    )


class LogEntrySourceLocation(proto.Message):
    r"""Additional information about the source code location that
    produced the log entry.

    Attributes:
        file (str):
            Optional. Source file name. Depending on the
            runtime environment, this might be a simple name
            or a fully-qualified name.
        line (int):
            Optional. Line within the source file.
            1-based; 0 indicates no line number available.
        function (str):
            Optional. Human-readable name of the function or method
            being invoked, with optional context such as the class or
            package name. This information may be used in contexts such
            as the logs viewer, where a file and line number are less
            meaningful. The format can vary by language. For example:
            ``qual.if.ied.Class.method`` (Java), ``dir/package.func``
            (Go), ``function`` (Python).
    """

    file: str = proto.Field(
        proto.STRING,
        number=1,
    )
    line: int = proto.Field(
        proto.INT64,
        number=2,
    )
    function: str = proto.Field(
        proto.STRING,
        number=3,
    )


__all__ = tuple(sorted(__protobuf__.manifest))
